import json
from . import exceptions

class PerspectivesClient():

    def __init__(self, client):
        self.client = client

    def list(self, org_id=None, active_only=False):
        uri = '/v1/perspective_schemas'
        params = [
            ('org_id', org_id),
            ('active_only', 1 if active_only else 0)
        ]

        perspectives = self.client.query(uri, method='GET', params=params)

        return perspectives


    def get(self, perspective_id, org_id=None, include_version=True):
        uri = f'/v1/perspective_schemas/{perspective_id}'
        params = [
            ('org_id', org_id),
            ('include_version', 1 if include_version else 0)
        ]

        perspective = self.client.query(uri, method='GET', params=params)

        return perspective


    def create(self, perspective_schema, org_id=None, include_version=True, client_api_id=None):
        uri = '/v1/perspective_schemas/'
        params = [
            ('org_id', org_id),
            ('include_version', 1 if include_version else 0),
            ('client_api_id', client_api_id)
        ]
        data = perspective_schema

        response = self.client.query(
            uri, method='POST', data=json.dumps(data), params=params
        )

        return response


    def duplicate(self, perspective_id, perspective_name, org_id=None, include_version=True, client_api_id=None):
        perspective_schema = self.get(
            perspective_id, org_id=org_id, include_version=include_version)
        perspective_schema['schema']['name'] = perspective_name

        response = self.create(
            perspective_schema, org_id=org_id, include_version=include_version, 
            client_api_id=client_api_id)

        return response


    def update(self, perspective_id, perspective_schema, org_id=None, include_version=True, check_version=None):
        uri = '/v1/perspective_schemas/{perspective_id}'
        params = [
            ('org_id', org_id),
            ('include_version', 1 if include_version else 0),
            ('check_version', check_version)
        ]
        data = perspective_schema

        response = self.client.query(
            uri, method='PUT', data=json.dumps(data), params=params
        )

        return response


    def delete(self, perspective_id, org_id=None, force=False, hard_delete=False):
        uri = '/v1/perspective_schemas/{perspective_id}'
        params = [
            ('org_id', org_id),
            ('force', 1 if force else 0),
            ('hard_delete', 1 if hard_delete else 0)
        ]

        response = self.client.query(uri, method='DELETE', params=params)

        return response



import requests
import os
import json

from cloudhealth import exceptions
from .assets import AssetsClient
from .perspectives import PerspectivesClient
from .reports import ReportingClient
from .sso import SsoClient
from .awsaccounts import AwsAccountsClient
from .metrics import MetricsClient
from .organization import OrganizationClient
from .tags import TagsClient
from .policies import PoliciesClient
from .partner import PartnerClient

class Client():
    def __init__(self,
                 endpoint,
                 api_key,
                 proxies=None):
        self.endpoint = endpoint
        self.api_key = api_key
        self.headers = {
            'Authorization': f'Bearer {self.api_key}',
            'Accept': 'application/json'
        }
        self.proxies = proxies


    def query(self,
            uri,
            data=None,
            params=None,
            headers={},
            method='GET'):

        url = f'{self.endpoint}{uri}'

        if len(url) >= 4000:
            raise exceptions.CloudHealthError(
                'The maximum permissible length of a URI string is 4000 characters.'
            )

        headers.update(self.headers)
        
        response = requests.request(
            method,
            url,
            data=json.dumps(data),
            params=params,
            headers=headers,
            proxies=self.proxies)

        if response.status_code not in [200, 201, 204]:
            error = response.json().get('error')
            raise exceptions.CloudHealthError(f'{response.status_code} - {error}')

        return response.json()


class CloudHealth():
    def __init__(self, endpoint='https://chapi.cloudhealthtech.com/', api_key=None, proxies=None):
        if not api_key:
            try:
                api_key = os.environ['CLOUDHEALTH_API_KEY']
            except KeyError:
                raise exceptions.CloudHealthError('API_KEY not set')

        self._client = Client(endpoint, api_key, proxies=proxies)
        self.assets = AssetsClient(self._client)
        self.perspectives = PerspectivesClient(self._client)
        self.reports = ReportingClient(self._client)
        self.sso = SsoClient(self._client)
        self.awsaccounts = AwsAccountsClient(self._client)
        self.metrics = MetricsClient(self._client)
        self.organization = OrganizationClient(self._client)
        self.tags = TagsClient(self._client)
        self.policies = PoliciesClient(self._client)
        self.partner = PartnerClient(self._client)



from . import exceptions

class TagsClient():

    def __init__(self, client):
        self.client = client

    def update(self, tag_groups):
        uri = '/v1/custom_tags'
        data = { "tag_groups": tag_groups }

        response = self.client.query(uri, method='POST', data=data)

        return response



from . import exceptions

class AwsAccountsClient():

    def __init__(self, client):
        self.client = client

    def enable(self, name, authentication, billing, cost_and_usage_report,
               cloudtrail, cloudwatch, tags, hide_public_fields=True,
               region='global', primary_aws_region='us-east-1', org_id=''):
        uri = '/v1/aws_accounts'
        params = [
            ('org_id', org_id),
        ]
        data = {
			"name": name,
		    "authentication": authentication,
			"billing": billing,
			"cost_and_usage_report" : cost_and_usage_report,
			"cloudtrail": cloudtrail,
			"tags": tags
        }
        account = self.client.query(
            uri, method='POST', data=data, params=params
        )

        return account


    def list(self, page=1, per_page=30, org_id=''):
        uri = '/v1/aws_accounts'
        params = [
            ('org_id', org_id),
            ('per_page', per_page),
            ('page',page)
        ]

        accounts = self.client.query(uri, method='GET', params=params)

        return accounts


    def get(self, account_id):
        uri = f'/v1/aws_accounts/{account_id}'

        account = self.client.query(uri, method='GET')

        return account


    def update(self, account_id, account_attributes):
        uri = f'/v1/aws_accounts/{account_id}'
        data = account_attributes

        account = self.client.query(uri, method='PUT', data=data)

        return account


    def delete(self, account_id):
        uri = f'/v1/aws_accounts/{account_id}'

        response = self.client.query(uri, method='DELETE')

        return response


    def get_external_id(self, account_id):
        uri = f'/v1/aws_accounts/{account_id}/generate_external_id'

        response = self.client.query(uri, method='GET')

        return response



from . import exceptions

class OrganizationClient():

    def __init__(self, client):
        self.client = client


    def list(self, page=None, per_page=None, org_id=None):
        uri = '/v2/organizations'
        params = []

        if page:
            params.append(('page', page))
        if per_page:
            params.append(('per_page', per_page))
        if org_id:
            params.append(('org_id', org_id))

        organizations = self.client.query(uri, method='GET', params=params)

        return organizations


    def update(self, org_id, description):
        uri = f'/v2/organizations/{org_id}'
        data = { "description": description }

        organization = self.client.query(
            uri, method='PUT', data=data
        )

        return organization


    def delete(self, org_id):
        uri = f'/v2/organizations/{org_id}'

        response = self.client.query(uri, method='DELETE')

        return response


    def create(self, name, description=None, parent_organization_id=None):
        uri = '/v2/organizations'
        data = { "name": name }

        if description:
            data['description'] = description
        if parent_organization_id:
            data['parent_organization_id'] = parent_organization_id

        organization = self.client.query(
            uri, method='POST', data=data
        )

        return organization


    def assign_accounts(self, org_id, aws_accounts=None, azure_subscriptions=None, gcp_compute_projects=None, data_center_accounts=None):
        uri = f'/v2/organizations/{org_id}'
        data = { "accounts": "add" }

        if not (aws_accounts or 
                azure_subscriptions or 
                gcp_compute_projects or
                data_center_accounts):
            raise exceptions.CloudHealthError('You must pass at least one of the parameters.')

        if aws_accounts:
            data['aws_accounts'] = aws_accounts
        if azure_subscriptions:
            data['azure_subscriptions'] = azure_subscriptions
        if gcp_compute_projects:
            data['gcp_compute_projects'] = gcp_compute_projects
        if data_center_accounts:
            data['data_center_accounts'] = data_center_accounts

        organization = self.client.query(
            uri, method='PUT', data=data
        )

        return organization


    def get_accounts(self, org_id, account_type, page=None, per_page=30, query=None, sort=None, is_down=False):
        uri = f'/v2/organizations/{org_id}/{account_type}'
        params = [
            ('per_page', per_page),
            ('is_down', is_down)
        ]

        if page:
            params.append(('page', page))
        if query:
            params.append(('query', query))
        if sort:
            params.append(('sort', sort))

        accounts = self.client.query(uri, method='GET', params=params)

        return accounts


    def get_allowed_accounts(self, org_id, account_type, per_page=30, page=None):
        uri = f'/v2/organizations/{org_id}/available_accounts'
        params = [
            ('type', account_type),
            ('per_page', per_page)
        ]

        if page:
            params.append(('page', page))

        accounts = self.client.query(uri, method='GET', params=params)

        return accounts


    def add_accounts_to_assignment(self, org_id, aws_accounts=None, azure_subscriptions=None, gcp_compute_projects=None, data_center_accounts=None):
        uri = f'/v2/organizations/{org_id}/accounts'
        data = { "accounts": "add" }

        if not (aws_accounts or 
                azure_subscriptions or 
                gcp_compute_projects or
                data_center_accounts):
            raise exceptions.CloudHealthError('You must pass at least one of the parameters.')

        if aws_accounts:
            data['aws_accounts'] = aws_accounts
        if azure_subscriptions:
            data['azure_subscriptions'] = azure_subscriptions
        if gcp_compute_projects:
            data['gcp_compute_projects'] = gcp_compute_projects
        if data_center_accounts:
            data['data_center_accounts'] = data_center_accounts

        assignment = self.client.query(
            uri, method='PATCH', data=data
        )

        return assignment


    def replace_assignment(self, org_id, aws_accounts=None, azure_subscriptions=None, gcp_compute_projects=None, data_center_accounts=None):
        uri = f'/v2/organizations/{org_id}/accounts'
        data = {}

        if not (aws_accounts or 
                azure_subscriptions or 
                gcp_compute_projects or
                data_center_accounts):
            raise exceptions.CloudHealthError('You must pass at least one of the parameters.')

        if aws_accounts:
            data['aws_accounts'] = aws_accounts
        if azure_subscriptions:
            data['azure_subscriptions'] = azure_subscriptions
        if gcp_compute_projects:
            data['gcp_compute_projects'] = gcp_compute_projects
        if data_center_accounts:
            data['data_center_accounts'] = data_center_accounts

        assignment = self.client.query(
            uri, method='PUT', data=data
        )

        return assignment


    def remove_assigned_accounts(self, org_id, aws_accounts=None, azure_subscriptions=None, gcp_compute_projects=None, data_center_accounts=None):
        uri = f'/v2/organizations/{org_id}/accounts'
        data = { "accounts": "remove" }

        if not (aws_accounts or
                azure_subscriptions or
                gcp_compute_projects or
                data_center_accounts):
            raise exceptions.CloudHealthError('You must pass at least one of the parameters.')

        if aws_accounts:
            data['aws_accounts'] = aws_accounts
        if azure_subscriptions:
            data['azure_subscriptions'] = azure_subscriptions
        if gcp_compute_projects:
            data['gcp_compute_projects'] = gcp_compute_projects
        if data_center_accounts:
            data['data_center_accounts'] = data_center_accounts

        accounts = self.client.query(
            uri, method='PATCH', data=data
        )

        return accounts



import json
from . import exceptions

class MetricsClient():

    def __init__(self, client):
        self.client = client

    def get(self, asset, granularity=None, _from=None, _to=None,
            time_range=None, page=None, per_page=None, org_id=None):
        uri = '/metrics/v1'
        params = [('asset', asset)]

        if granularity:
            params.append(('granularity', granularity))
        if _from:
            params.append(('from', _from))
        if _to:
            params.append(('to', _to))
        if time_range:
            params.append(('time_range', time_range))
        if page:
            params.append(('page', page))
        if per_page:
            params.append(('per_page', per_page))
        if org_id:
            params.append(('org_id', org_id))

        metrics = self.client.query(uri, method='GET', params=params)

        return metrics


    def upload(self, payload, dryrun=False, org_id=None):
        uri = '/metrics/v1'
        params = [('dryrun', dryrun)]

        if org_id:
            params.append(('org_id', org_id))

        data = payload

        response = self.client.query(
            uri, method='POST', data=json.dumps(data), params=params
        )

        return response



from . import exceptions

class PoliciesClient():

    def __init__(self, client):
        self.client = client

    def list(self, client_api_id=None, page=1, per_page=30):
        uri = '/v1/policies'
        params = [
            ('page', page),
            ('per_page', per_page)
        ]

        if client_api_id:
            params.append(('api_key', client_api_id))

        policies = self.client.query(uri, method='GET', params=params)

        return policies


    def list_blocks(self, policy_id, client_api_id=None, page=1, per_page=30):
        uri = f'/v1/policies/{policy_id}/policy_blocks'
        params = [
            ('page', page),
            ('per_page', per_page)
        ]

        if client_api_id:
            params.append(('api_key', client_api_id))

        policy_blocks = self.client.query(uri, method='GET', params=params)

        return policy_blocks


    def list_violations(self, policy_id, policy_block_id, client_api_id=None, page=1, per_page=30, count=1):
        uri = f'/v1/policies/{policy_id}/policy_blocks/{policy_block_id}/violations'
        params = [
            ('page', page),
            ('per_page', per_page),
            ('count', count)
        ]

        if client_api_id:
            params.append(('api_key', client_api_id))

        policy_violations = self.client.query(uri, method='GET', params=params)

        return policy_violations


    def get_violation(self, policy_id, policy_block_id, violation_id, client_api_id=None, page=1, per_page=100):
        uri = f'/v1/policies/{policy_id}/policy_blocks/{policy_block_id}/violations/{violation_id}'
        params = [
            ('page', page),
            ('per_page', per_page)
        ]

        if client_api_id:
            params.append(('api_key', client_api_id))

        policy_violation = self.client.query(uri, method='GET', params=params)

        return policy_violation



from datetime import datetime
from . import exceptions

class PartnerClient():

    def __init__(self, client):
        self.client = client

    def get_report_data(self, report_type, report_id, client_api_id):
        uri = f'olap_reports/{report_type}/{report_id}'
        params = [('client_api_id', client_api_id)]

        report = self.client.query(uri, method='GET', params=params)

        return report


    def list_assets(self, client_api_id, name):
        uri = '/api/search.json'
        params = [
            ('api_version', 2),
            ('client_api_id', client_api_id),
            ('name', name)
        ]

        assets = self.client.query(uri, method='GET', params=params)

        return assets


    def create_customer(self, name=None, street1=None, street2=None,city=None, state=None, zipcode=None, country=None, classification=None,trial_expiration=None, billing_contact=None,partner_billing_configuration=False,partner_billing_configuration_folder=None, tags=None):
        uri = '/v1/customers'
        data = {
            "name": name,
            "address": {
                "street1": street1,
                "street2": street2,
                "city": city,
                "state": state,
                "zipcode": zipcode,
                "country": country
            },
            "partner_billing_configuration": {}
        }

        if classification:
            data['classification'] = classification
        if isinstance(trial_expiration, datetime):
            data['trial_expiration'] = trial_expiration.isoformat()
        if isinstance(trial_expiration, str):
            data['trial_expiration'] = trial_expiration
        if billing_contact:
            data['billing_contact'] = billing_contact
        if partner_billing_configuration:
            data['partner_billing_configuration']['enabled'] = partner_billing_configuration
        if partner_billing_configuration_folder:
            data['partner_billing_configuration']['folder'] = partner_billing_configuration_folder
        if tags:
            data['tags'] = tags

        customer = self.client.query(uri, method='POST', data=data)

        return customer


    def update_customer(self, client_api_id, name=None, street1=None, street2=None, city=None, state=None, zipcode=None, country=None, classification=None, trial_expiration=None, billing_contact=None, partner_billing_configuration=False, partner_billing_configuration_folder=None, tags=None):
        uri = f'/v1/customers/{client_api_id}'
        data = {
            "partner_billing_configuration": {},
            "address": {}
        }

        if name:
            data['name'] = name
        if street1:
            data['street1'] = street1
        if street2:
            data['street2'] = street2
        if city:
            data['city'] = city
        if state:
            data['state'] = state
        if zipcode:
            data['zipcode'] = zipcode
        if country:
            data['country'] = country
        if classification:
            data['classification'] = classification
        if isinstance(trial_expiration, datetime):
            data['trial_expiration'] = trial_expiration.isoformat()
        if isinstance(trial_expiration, str):
            data['trial_expiration'] = trial_expiration
        if billing_contact:
            data['billing_contact'] = billing_contact
        if partner_billing_configuration:
            data['partner_billing_configuration']['enabled'] = partner_billing_configuration
        if partner_billing_configuration_folder:
            data['partner_billing_configuration']['folder'] = partner_billing_configuration_folder
        if tags:
            data['tags'] = tags

        if not data['address']:
            del data['address']
        if not data['partner_billing_configuration']:
            del data['partner_billing_configuration']


        customer = self.client.query(uri, method='PUT', data=data)

        return customer


    def get_customer(self, client_api_id):
        uri = f'/v1/customers/{client_api_id}'

        customer = self.client.query(uri, method='GET')

        return customer


    def delete_customer(self, client_api_id):
        uri = f'/v1/customers/{client_api_id}'
        
        response = self.client.query(uri, method='DELETE')

        return response

    
    def list_customers(self, page=1, per_page=30):
        uri = f'/v1/customers'
        params = [
            ('page', page),
            ('per_page', per_page)
        ]

        customers = self.client.query(uri, method='GET', params=params)

        return customers


    def get_statement(self, client_api_id, status=None, billing_period=None, page=1, per_page=30):
        uri = '/v1/customer_statements'
        params = [
            ('client_api_id', client_api_id),
            ('page', page),
            ('per_page', per_page)
        ]

        if status:
            params.append(('status', status))
        if billing_period:
            params.append(('billing_period', billing_period))

        statement = self.client.query(uri, method='GET', params=params)

        return statementa


    def list_statements(self, status=None, billing_period=None, page=1, per_page=30):
        uri = '/v1/customer_statements'
        params = [
            ('page', page),
            ('per_page', per_page)
        ]

        if status:
            params.append(('status', status))
        if billing_period:
            params.append(('billing_period', billing_period))

        statement = self.client.query(uri, method='GET', params=params)

        return statement


    def connect_govcloud_account(self, client_api_id, govcloud_acct_id, commercial_acct_id):
        uri = '/api/v1/govcloud_linkages'
        params = [('client_api_id', client_api_id)]
        data = {
            "govcloud_acct_id": govcloud_acct_id,
            "commercial_acct_id": commercial_acct_id
        }

        linkage = self.client.query(uri, method='POST', params=params, data=data)

        return linkage


    def list_linkages(self, client_api_id):
        uri = '/api/v1/govcloud_linkages'
        params = [('client_api_id', client_api_id)]

        linkages = self.client.query(uri, method='GET', params=params)

        return linkages


    def get_linkage(self, client_api_id, linkage_id):
        uri = f'/api/v1/govcloud_linkages/{linkage_id}'
        params = [('client_api_id', client_api_id)]

        linkage = self.client.query(uri, method='GET', params=params)

        return linkage
